@@ -14,7 +14,8 @@ from mobio.libs.Singleton import Singleton
 from mobio.libs.caching import LruCache
 
 from .config import ConsumerTopic, MobioEnvironment, StoreCacheType, Cache
-from .constant import DirSaveFile, FieldConstantResponse, CONSTANT_MAP_FORMAT_FILE
+from .constant import DirSaveFile, FieldConstantResponse, CONSTANT_MAP_FORMAT_FILE, ParamMessageSaveInfoFile, \
+    ParamMessageUploadMediaSdk
 from .custom_mimetypes import CustomMimetypes
 from .dir import APP_TMP_DATA_DIR, STATIC_DATA_DIR
 
@@ -71,11 +72,11 @@ class MobioMediaSDK(object):
         self.instance = Producer(conf)
 
     @staticmethod
-    def valid_input_upload(file_path, file_data, merchant_id, expire=None):
+    def valid_input_upload(file_path, file_data, file_byte, merchant_id, expire=None):
         if not merchant_id:
             raise Exception("[ERROR] merchant_id require")
-        if not file_path and not file_data:
-            raise Exception("[ERROR] Need to pass 1 of 2 parameters file_path or file_data")
+        if not file_path and not file_data and not file_byte:
+            raise Exception("[ERROR] Need to pass 1 of 3 parameters file_path or file_data or file_byte")
         if file_path and not os.path.isfile(file_path):
             raise Exception("[ERROR] file_path: {} not exist.".format(file_path))
         if expire:
@@ -103,6 +104,7 @@ class MobioMediaSDK(object):
             merchant_id: str,
             file_path: str = '',
             file_data=None,
+            file_byte=None,
             filename: str = '',
             type_media=DirSaveFile.UPLOAD,
             tag: str = '',
@@ -115,6 +117,7 @@ class MobioMediaSDK(object):
         :param merchant_id:
         :param file_path:
         :param file_data:
+        :param file_byte:
         :param filename:
         :param type_media:
         :param tag:
@@ -132,6 +135,7 @@ class MobioMediaSDK(object):
         MobioMediaSDK.valid_input_upload(
             file_path=file_path,
             file_data=file_data,
+            file_byte=file_byte,
             merchant_id=merchant_id,
             expire=expire
         )
@@ -153,22 +157,24 @@ class MobioMediaSDK(object):
         if file_path:
             # shutil.move(file, dist_file)
             shutil.move(file_path, dist_file)
-        else:
+        elif file_data:
             file_data.save(dist_file)
+        else:
+            with open(dist_file, "wb") as f:
+                f.write(file_byte)
         url = os.path.join(host_by_merchant_id, self.folder_static, merchant_id, type_media, filename)
         if short_link:
             url = os.path.join(host_by_merchant_id, self.folder_static, filename)
         logger.debug("upload_without_kafka()::url:: %s" % (time.time() - time_start))
         data_send_producer = {
-            "filename": filename,
-            "merchant_id": merchant_id,
-            "url": url,
-            "type_media": type_media,
-            "tag": tag,
-            "expire": expire,
-            "dist_file": dist_file,
-            "mimetype_str": mimetype_str,
-            "do_not_delete": do_not_delete
+            ParamMessageSaveInfoFile.FILENAME: filename,
+            ParamMessageSaveInfoFile.URL: url,
+            ParamMessageSaveInfoFile.TYPE_MEDIA: type_media,
+            ParamMessageSaveInfoFile.TAG: tag,
+            ParamMessageSaveInfoFile.EXPIRE: expire,
+            ParamMessageSaveInfoFile.DIST_FILE: dist_file,
+            ParamMessageSaveInfoFile.MIMETYPE_STR: mimetype_str,
+            ParamMessageSaveInfoFile.DO_NOT_DELETE: do_not_delete
         }
 
         self.send_message_to_topic(topic=ConsumerTopic.TOPIC_SAVE_INFO_MEDIA_SDK, data=json.dumps(data_send_producer))
@@ -185,6 +191,7 @@ class MobioMediaSDK(object):
             merchant_id,
             file_path: str = None,
             file_data=None,
+            file_byte=None,
             filename=None,
             type_media=DirSaveFile.UPLOAD,
             tag=None,
@@ -197,6 +204,7 @@ class MobioMediaSDK(object):
         :param merchant_id:
         :param filename:
         :param file_path:
+        :param file_byte:
         :param file_data:
         :param filename:
         :param type_media:
@@ -213,6 +221,7 @@ class MobioMediaSDK(object):
         MobioMediaSDK.valid_input_upload(
             file_path=file_path,
             file_data=file_data,
+            file_byte=file_byte,
             merchant_id=merchant_id,
             expire=expire
         )
@@ -233,8 +242,11 @@ class MobioMediaSDK(object):
         if file_path:
             # shutil.move(file, tmp_file)
             shutil.move(file_path, tmp_file)
-        else:
+        elif file_data:
             file_data.save(tmp_file)
+        else:
+            with open(tmp_file, "wb") as f:
+                f.write(file_byte)
         logger.debug("upload_with_kafka()::move file:: %s" % (time.time() - time_start))
         dist_file, filename = self.create_dir_save_file(
             folder=STATIC_DATA_DIR,
@@ -249,16 +261,16 @@ class MobioMediaSDK(object):
         if short_link:
             url = os.path.join(host_by_merchant_id, self.folder_static, filename)
         data_send_producer = {
-            "filename": filename,
-            "merchant_id": merchant_id,
-            "tmp_file": tmp_file,
-            "type_media": type_media,
-            "tag": tag,
-            "expire": expire,
-            "dist_file": dist_file,
-            "mimetype_str": mimetype_str,
-            "do_not_delete": do_not_delete,
-            "url": url
+            ParamMessageUploadMediaSdk.FILENAME: filename,
+            ParamMessageUploadMediaSdk.MERCHANT_ID: merchant_id,
+            ParamMessageUploadMediaSdk.TMP_FILE: tmp_file,
+            ParamMessageUploadMediaSdk.TYPE_MEDIA: type_media,
+            ParamMessageUploadMediaSdk.TAG: tag,
+            ParamMessageUploadMediaSdk.EXPIRE: expire,
+            ParamMessageUploadMediaSdk.DIST_FILE: dist_file,
+            ParamMessageUploadMediaSdk.MIMETYPE_STR: mimetype_str,
+            ParamMessageUploadMediaSdk.DO_NOT_DELETE: do_not_delete,
+            ParamMessageUploadMediaSdk.URL: url
         }
         self.send_message_to_topic(topic=ConsumerTopic.TOPIC_UPLOAD_MEDIA_SDK, data=json.dumps(data_send_producer))
         logger.debug("upload_with_kafka()::send message to topic:: %s" % (time.time() - time_start))
@@ -269,6 +281,66 @@ class MobioMediaSDK(object):
             FieldConstantResponse.FORMAT: mimetype_str,
         }
 
+    def create_public_url_without_file(
+            self,
+            merchant_id,
+            filename=None,
+            type_media=DirSaveFile.UPLOAD,
+            tag=None,
+            expire=None,
+            do_not_delete=False,
+            short_link=False,
+            mimetype_str=None
+    ):
+        dist_file, filename = self.create_dir_save_file(
+            folder=STATIC_DATA_DIR,
+            merchant_id=merchant_id,
+            type_media=type_media,
+            filename=filename,
+            short_link=short_link
+        )
+        host_by_merchant_id = self._get_host_by_merchant_id(merchant_id=merchant_id)
+        logger.info("create_public_url_without_file :: host_by_merchant_id :: %s " % host_by_merchant_id)
+        url = os.path.join(host_by_merchant_id, self.folder_static, merchant_id, type_media, filename)
+        if short_link:
+            url = os.path.join(host_by_merchant_id, self.folder_static, filename)
+        logger.info("create_public_url_without_file :: url :: %s " % url)
+        logger.info("create_public_url_without_file :: local_path :: %s " % dist_file)
+        time_start = time.time()
+        data_send_producer = {
+            ParamMessageSaveInfoFile.FILENAME: filename,
+            ParamMessageSaveInfoFile.MERCHANT_ID: merchant_id,
+            ParamMessageSaveInfoFile.URL: url,
+            ParamMessageSaveInfoFile.TYPE_MEDIA: type_media,
+            ParamMessageSaveInfoFile.TAG: tag,
+            ParamMessageSaveInfoFile.EXPIRE: expire,
+            ParamMessageSaveInfoFile.DIST_FILE: dist_file,
+            ParamMessageSaveInfoFile.DO_NOT_DELETE: do_not_delete,
+            ParamMessageSaveInfoFile.MIMETYPE_STR: mimetype_str,
+            ParamMessageSaveInfoFile.NOT_CHECK_PATH_EXISTS: True
+        }
+
+        self.send_message_to_topic(topic=ConsumerTopic.TOPIC_SAVE_INFO_MEDIA_SDK, data=json.dumps(data_send_producer))
+        logger.debug("upload_without_kafka()::send message to topic:: %s" % (time.time() - time_start))
+        return {
+            FieldConstantResponse.URL: url,
+            FieldConstantResponse.LOCAL_PATH: dist_file,
+            FieldConstantResponse.FILENAME: filename
+        }
+
+    def finish_save_file_by_public_url(self, filepath=None, file_data=None, file_byte=None, url=None):
+        if not url:
+            raise Exception("[ERROR] :: url not None")
+        local_path = self.get_path_by_url(url, is_check_path_exists=False)
+        logger.info("finish_save_file_by_url :: get_path_by_url % s result :: %s " % (url, local_path))
+        if filepath:
+            shutil.move(filepath, local_path)
+        elif file_data:
+            file_data.save(local_path)
+        else:
+            with open(local_path, "wb") as f:
+                f.write(file_byte)
+
     @classmethod
     def create_dir_save_file(cls, folder, merchant_id, type_media, filename, short_link=False):
         dir_file = os.path.join(folder, merchant_id)
@@ -295,7 +367,8 @@ class MobioMediaSDK(object):
 
     def get_path_by_url(
             self,
-            url
+            url,
+            is_check_path_exists=True
     ):
         if not url:
             raise ValueError("[ERROR] URL not none")
@@ -304,7 +377,7 @@ class MobioMediaSDK(object):
         except Exception as ex:
             logger.error("[ERROR] get local_path_file error %s " % ex)
             raise Exception("url %s not found" % url)
-        if not os.path.isfile(local_path):
+        if is_check_path_exists and not os.path.isfile(local_path):
             raise Exception("[ERROR] local_path :: %s not exists" % local_path)
         return local_path
 
@@ -331,42 +404,6 @@ class MobioMediaSDK(object):
             filename=filename
         )
 
-    def finish_save_file_by_filepath(
-            self,
-            filepath,
-            do_not_delete: bool = False,
-            tag: str = "",
-            expire: str = ''
-    ):
-        if not os.path.isfile(filepath):
-            raise FileExistsError
-        dist_file = filepath
-
-        filename, file_extension = os.path.splitext(filepath)
-        filepath = filepath.split("/")
-        filepath = [path for path in filepath if path]
-        filename = filepath[-1]
-        merchant_id = filepath[-3]
-        type_media = self.check_type_media(filepath[-2])
-
-        host_by_merchant_id = self._get_host_by_merchant_id(merchant_id=merchant_id)
-        url = os.path.join(host_by_merchant_id, self.folder_static, merchant_id, type_media, filename)
-        data_send_producer = {
-            "filename": filename,
-            "merchant_id": merchant_id,
-            "url": url,
-            "type_media": type_media,
-            "tag": tag,
-            "expire": expire,
-            "dist_file": dist_file,
-            "mimetype_str": file_extension,
-            "do_not_delete": do_not_delete
-        }
-
-        self.send_message_to_topic(topic=ConsumerTopic.TOPIC_SAVE_INFO_MEDIA_SDK, data=json.dumps(data_send_producer))
-
-        return url
-
     def check_type_media(self, type_media):
         if type_media not in [DirSaveFile.UPLOAD, DirSaveFile.DOWNLOAD]:
             raise ValueError("Type media is not correct!")
@@ -401,11 +438,13 @@ class MobioMediaSDK(object):
             url="",
             file_path=None,
             file_data=None,
+            file_byte=None,
             desired_format=None
     ):
         MobioMediaSDK.valid_input_upload(
             file_path=file_path,
             file_data=file_data,
+            file_byte=file_byte,
             merchant_id=merchant_id
         )
         mimetype_str = MobioMediaSDK.get_mimetype(file_path=file_path, file_data=file_data)
@@ -450,10 +489,6 @@ class MobioMediaSDK(object):
             "urls": urls,
             "type_delete": type_delete
         }
-
-        # message_data = Message(message=data_send_producer)
-        # KafkaUtil().produce(topic=ConsumerTopic.TOPIC_DELETE_MEDIA_SDK, message=message_data)
-
         self.send_message_to_topic(
             topic=ConsumerTopic.TOPIC_DELETE_MEDIA_SDK,
             data=json.dumps(data_send_producer)
